#!/bin/bash

if [ $# -ne 1 ]; then
    echo "You need to give application to test on commandline"
    exit 1
fi

EXECPATH=../..
EXEC=$1
TMPFILE=/tmp/testout

f_grep() {
    ARG="$1"
    if [ `grep "${ARG}" ${TMPFILE} | wc -l` == "0" ]; then
        return 1
    fi
    return 0
}

f_ngrep() {
    ARG="$1"
    if [ `grep "${ARG}" ${TMPFILE} | wc -l` != "0" ]; then
        return 1
    fi
    return 0
}

f_listlen() {
    LIST=$(cat ${TMPFILE})
    DELIM=$(echo ${1} | cut -d ' ' -f 1)
    COUNT=$(echo ${1} | cut -d ' ' -f 2)
    CHARS=${LIST//[^${DELIM}]}
    LENGTH=$(expr ${#CHARS} + 1)
    if [ ${LENGTH} != "${COUNT}" ]; then
        return 1
    fi
    return 0
}

if [ ! -e ${EXEC}.txt ]; then
    echo "Cannot find testfile ${EXEC}.txt"
    exit 1
fi

while read -r LINE || [[ -n $LINE ]]; do
    if [ -z "${LINE}" ]; then continue; fi
    if [[ "${LINE}" =~ \#.* ]]; then continue; fi

    OPTIONS=$(echo "${LINE}" | cut -d '|' -f 1)
    RESULTS=$(echo "${LINE}" | cut -d '|' -f 2-)
    NUM_RESULTS="${RESULTS//[^|]}"
    EXITCODE=$(${EXECPATH}/${EXEC} ${OPTIONS} 1>${TMPFILE} 2>&1  ; echo $?)
    STATE=0
    for ((i=1;i<=${#NUM_RESULTS}+1;i++)); do
        RESULT=$(echo ${RESULTS} | cut -d '|' -f ${i})
        RESULT_CMD=$(echo $RESULT | cut -d' ' -f1)
        RESULT_OPTS=$(echo $RESULT | cut -d ' ' -f 2-)
        if [ ${RESULT_CMD} == "EXIT" ]; then
            if [ "${RESULT_OPTS}" != "$EXITCODE" ]; then
                STATE=1
            fi
        elif [ ${RESULT_CMD} == "GREP" ]; then
            f_grep "${RESULT_OPTS}"
            STATE=$?
        elif [ ${RESULT_CMD} == "NGREP" ]; then
            f_ngrep "${RESULT_OPTS}"
            STATE=$?
        elif [ ${RESULT_CMD} == "LISTLEN" ]; then
            f_listlen "${RESULT_OPTS}"
            STATE=$?
        fi
    done
    if [ $STATE -eq 0 ]; then
        echo "SUCCESS : ${EXEC}" "${OPTIONS}"
    else
        echo "FAIL : ${EXEC}" "${OPTIONS}"
    fi
done < ${EXEC}.txt


